"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const crypto = require("crypto");
const getStream = require("get-stream");
const request = require("request");
/** return the sha256 of a file as a hex-formatted string */
function sha256OfFile(path) {
    return __awaiter(this, void 0, void 0, function* () {
        const input = fs.createReadStream(path);
        const cipher = crypto.createHash('sha256');
        return yield getStream(input.pipe(cipher), { encoding: 'hex' });
    });
}
exports.sha256OfFile = sha256OfFile;
/** download a URL and save it to a file */
function downloadUrlToFile(url, path) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileStream = fs.createWriteStream(path);
        yield new Promise((res, rej) => {
            request(url).pipe(fileStream).on('close', () => {
                res();
            });
        });
    });
}
exports.downloadUrlToFile = downloadUrlToFile;
function readFileWithDefault(path, defaultContent) {
    try {
        return fs.readFileSync(path, 'utf8');
    }
    catch (e) {
        if (!fs.existsSync(path))
            return defaultContent;
        throw e;
    }
}
exports.readFileWithDefault = readFileWithDefault;
function readJsonFileWithDefault(path, defaultContent) {
    try {
        return JSON.parse(fs.readFileSync(path, 'utf8'));
    }
    catch (e) {
        if (!fs.existsSync(path))
            return defaultContent;
        throw e;
    }
}
exports.readJsonFileWithDefault = readJsonFileWithDefault;
function writeJsonFile(path, value, indent = true) {
    if (indent === true)
        indent = '    ';
    fs.writeFileSync(path, JSON.stringify(value, null, indent));
}
exports.writeJsonFile = writeJsonFile;
function patchJsonFile(path, _a, _b) {
    let [opts = {}, callback] = [_a, _b];
    if (!callback)
        [opts, callback] = [{}, _a];
    const { defaultValue = null, indent = false } = opts;
    const value = readJsonFileWithDefault(path, defaultValue);
    let newValue = callback(value);
    if (newValue === undefined)
        newValue = value;
    writeJsonFile(path, newValue, indent);
}
exports.patchJsonFile = patchJsonFile;
